# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tw_hooks', 'tw_hooks.base_hooks', 'tw_hooks.hooks', 'tw_hooks.scripts']

package_data = \
{'': ['*']}

install_requires = \
['bubop==0.1.8']

entry_points = \
{'console_scripts': ['install-hook-shims = '
                     'tw_hooks.scripts.install_hook_shims:main']}

setup_kwargs = {
    'name': 'tw-hooks',
    'version': '0.1.5',
    'description': 'Collection of Taskwarrior hooks',
    'long_description': '# Taskwarrior Hooks\n\n<p align="center">\n  <img src="https://raw.githubusercontent.com/bergercookie/tw-hooks/master/misc/logo.png"/>\n</p>\n\n<a href="https://github.com/bergercookie/tw-hooks/actions" alt="CI">\n<img src="https://github.com/bergercookie/tw-hooks/actions/workflows/ci.yml/badge.svg" /></a>\n<a href="https://github.com/pre-commit/pre-commit">\n<img src="https://img.shields.io/badge/pre--commit-enabled-brightgreen?logo=pre-commit&logoColor=white" alt="pre-commit"></a>\n\n<a href=\'https://coveralls.io/github/bergercookie/tw-hooks?branch=master\'>\n<img src=\'https://coveralls.io/repos/github/bergercookie/tw-hooks/badge.svg?branch=master\' alt=\'Coverage Status\' /></a>\n<a href="https://github.com/bergercookie/tw-hooks/blob/master/LICENSE.md" alt="LICENSE">\n<img src="https://img.shields.io/github/license/bergercookie/tw-hooks.svg" /></a>\n<a href="https://pypi.org/project/tw_hooks/" alt="pypi">\n<img src="https://img.shields.io/pypi/pyversions/tw-hooks.svg" /></a>\n<a href="https://badge.fury.io/py/tw-hooks">\n<img src="https://badge.fury.io/py/tw-hooks.svg" alt="PyPI version" height="18"></a>\n<a href="https://pepy.tech/project/tw-hooks">\n<img alt="Downloads" src="https://pepy.tech/badge/tw_hooks"></a>\n<a href="https://github.com/psf/black">\n<img alt="Code style: black" src="https://img.shields.io/badge/code%20style-black-000000.svg"></a>\n\n## Description\n\nThis is a collection of [Taskwarrior\nhooks](https://taskwarrior.org/docs/hooks_guide.html) that I use in my\nday-to-day workflows. It comes along a detection and easy-registration mechanism\nthat should make it easy to develop and then distribute your own hooks. The\nhooks are structured as classes under the `tw_hooks/hooks` directory.\n\n## Installation\n\nInstall it from `PyPI`:\n\n```sh\npip3 install --user --upgrade tw_hooks\n```\n\nTo get the latest version install directly from source:\n\n```sh\npip3 install --user --upgrade git+https://github.com/bergercookie/tw-hooks\n```\n\nAfter the installation, you have to run the `install-hooks-shims` executable\n(which by this point should be in your `$PATH`). Running it will create shims\n(thin wrapper scripts) under `~/.task/hooks` in order to register all the hooks\nwith Taskwarrior.\n\n## Available hooks\n\nCurrently the following hooks are available out-of-the-box:\n\n<table style="undefined;table-layout: fixed; width: 823px">\n<thead>\n  <tr>\n    <th>Hook</th>\n    <th>Description</th>\n    <th>Events</th>\n  </tr>\n</thead>\n<tbody>\n  <tr>\n    <td><tt>AutoTagBasedOnTags</tt></td>\n    <td>Inspect the list of tags in the added/modified tasks provided and add additional tags if required</td>\n    <td><tt>on-modify</tt>, <tt>on-add</tt></td>\n  </tr>\n  <tr>\n    <td><tt>CorrectTagNames</tt></td>\n    <td>Change tag names based on a predefined lookup table</td>\n    <td><tt>on-modify</tt>, <tt>on-add</tt></td>\n  </tr>\n  <tr>\n    <td><tt>DetectMutuallyExclusiveTags</tt></td>\n    <td>See whether the user has specified an incompatible combination of tags</td>\n    <td><tt>on-modify</tt>, <tt>on-add</tt></td>\n  </tr>\n  <tr>\n    <td><tt>PostLatestStartToI3Status</tt></td>\n    <td>When a task is started, send the title of the task to i3status-rs via DBus</td>\n    <td><tt>on-modify</tt></td>\n  </tr>\n  <tr>\n    <td><tt>WarnOnTaskCongestion</tt></td>\n    <td>Warn the user if there are too many tasks (due:today)</td>\n    <td><tt>on-exit</tt></td>\n  </tr>\n</tbody>\n</table>\n\n## Structure of a Hook\n\nThe purpose of this package is to facilitate the development and distribution of\nTaskwarrior hooks. To this purpose `install-hooks-shims` allows you to easily\nregister your own hooks, without having to manually copy items over to the\ntaskwarrior hooks location. `install-hooks-shims` will install a shim which will\ncall your hook automatically when required.\n\nThis is an example of a Taskwarrior hook that will be executed on Taskwarrior\nexit:\n\n```python\nfrom tw_hooks import OnExitHook\nclass WarnOnTaskCongestion(OnExitHook):\n    """Warn the user if there are too many tasks."""\n    def _on_exit(self, _):  # <--- Mandatory to implement this signature\n      # ...\n      return 0\n```\n\nAssuming that this hook is in a module called `warn_on_task_congestion.py` and\nthat the directory of this module is in your python path (e.g., by adding it\nexplicitly to `$PYTHONPATH`), then you can run the following to register your\nhook with taskwarrior:\n\n```sh\ninstall-hooks-shims -r warn_on_task_congestion\n```\n\nDuring your next Taskwarrior operation, if there are too many due:today tasks,\nyou should see something like this:\n\n```sh\nt add +test kalimera\nCreated task 719.\n[WarnOnTaskCongestion] Too many due:today tasks [threshold=9]\n```\n\n## Hooks API\n\nSubclass one of the following base hooks, and your method is going to be called\nduring that event:\n\n- [`OnAddHook`](https://github.com/bergercookie/tw-hooks/blob/master/tw_hooks/base_hooks/on_add_hook.py)\n  - Implement the `_on_add(self, added_task: TaskT)` method.\n- [`OnExitHook`](https://github.com/bergercookie/tw-hooks/blob/master/tw_hooks/base_hooks/on_exit_hook.py)\n  - Implement the `_on_exit(self, added_modified_tasks: List[TaskT])` method.\n- [`OnLaunchHook`](https://github.com/bergercookie/tw-hooks/blob/master/tw_hooks/base_hooks/on_launch_hook.py)\n  - Implement the `_on_launch(self)` method.\n- [`OnModifyHook`](https://github.com/bergercookie/tw-hooks/blob/master/tw_hooks/base_hooks/on_modify_hook.py)\n  - Implement the `_on_modify(self, original_task: TaskT, modified_task: TaskT)`\n    method.\n\n## Usage instructions for `install-hooks-shims`\n\n<!-- START sniff-and-replace install-hook-shims --help START -->\n\n```python\nusage: Detect Taskwarrior hooks and register an executable shim for each one of them.\n       [-h] [-t TASK_DIR] [-a] [-l]\n       [-r REGISTER_ADDITIONAL [REGISTER_ADDITIONAL ...]]\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -t TASK_DIR, --task-dir TASK_DIR\n                        Path to the taskwarrior main directory\n  -a, --all-hooks       Install shims for all the hooks\n  -l, --list-hooks      List the available hooks and exit\n  -r REGISTER_ADDITIONAL [REGISTER_ADDITIONAL ...], --register-additional REGISTER_ADDITIONAL [REGISTER_ADDITIONAL ...]\n\nUsage examples:\n===============\n\n- Install only the WarnOnTaskCongestion hook (assuming you\'ve installed tw_hooks with e.g., pip3)\n  install-hook-shims -r tw_hooks.hooks.warn_on_task_congestion\n\n- Install all the available hooks from this repo (assuming you\'ve installed tw_hooks with e.g., pip3)\n  install-hook-shims --all-hooks\n\n- Install a custom hook defined in .../dir/mod/hook_name.py. "dir" should be in your PYTHONPATH\n  install-hook-shims -r mod.hook_name\n\n- List all the available hooks and exit\n  install-hook-shims --list-hooks\n\n```\n\n<!-- END sniff-and-replace -->\n\n## Miscellaneous\n\n- [Contributing Guide](CONTRIBUTING.md)\n\n## FAQ\n\n- Why should I use this over raw taskwarrior hooks?\n  - Because this package does the heavy lifting pre-processing the input tasks\n    from the command line. It does so in a robust manner making sure it does\n    the right thing regardless of weather one or two commands are provided and\n    being robust to errors (e.g., `utf-8` decoding errors).\n  - It takes care to make the hooks fail safely even if it can\'t find required\n    modules (e.g., if you try invoking `task` from inside a `virtualenv` where\n    `tw-hooks` is not importable.\n  - It gives you a class-oriented approach and lets you install multiple hooks\n    from the same class, thus allowing these hooks to share common\n    configuration.\n  - It allows you to keep all your hooks together and keep\n    them as a package in some other place in your filesystem, e.g., in your\n    dotfiles and automatically adds the right glue-code so that Taskwarrior your\n    scripts without having to explicitly place it in `~/.task/hooks` or\n    symlinking it.\n\n## Self Promotion\n\nIf you find this tool useful, please [star it on\nGithub](https://github.com/bergercookie/tw-hooks)\nand consider donating.\n\n## Support\n\nIf something doesn\'t work, feel free to open an issue. You can also find me in\nthe [#taskwarrior Libera Chat](https://matrix.to/#/#taskwarrior:libera.chat).\n\n## TODO List\n\nSee [ISSUES\nlist](https://github.com/bergercookie/tw-hooks/issues)\nfor the things that I\'m currently either working on or interested in\nimplementing in the near future. In case there\'s something you are interesting\nin working on, don\'t hesitate to either ask for clarifications or just do it and\ndirectly make a PR.\n',
    'author': 'Nikos Koukis',
    'author_email': 'nickkouk@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/bergercookie/tw-hooks',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
